# -*- coding: utf-8 -*-
import oauth2 as oauth

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME

from ..utils import reverse_with_host

class SSOClient(oauth.Client):

    request_token_url = '%(HOST)s/%(REQUEST_TOKEN_PATH)s' % settings.PASSAPORTE_WEB
    access_token_url = '%(HOST)s/%(ACCESS_TOKEN_PATH)s' % settings.PASSAPORTE_WEB
    authorization_url = '%(HOST)s/%(AUTHORIZATION_PATH)s' % settings.PASSAPORTE_WEB
    user_data_url = '%(HOST)s/%(FETCH_USER_DATA_PATH)s' % settings.PASSAPORTE_WEB

    def __init__(self, *args, **kwargs):

        self.consumer = oauth.Consumer(
            settings.PASSAPORTE_WEB['CONSUMER_TOKEN'],
            settings.PASSAPORTE_WEB['CONSUMER_SECRET']
        )

        return super(SSOClient, self).__init__(self.consumer, *args, **kwargs)


    def fetch_request_token(self, callback_url=None):

        if callback_url is None:
            callback_url = reverse_with_host('sso_consumer:callback')

        resp, content = self.post(
            self.request_token_url, body='oauth_callback={0}'.format(callback_url)
        )

        if not str(resp.get('status')) == '200':
            raise AssertionError(resp, content)

        return oauth.Token.from_string(content)


    def fetch_access_token(self):

        resp, content = self.post(self.access_token_url)

        if not str(resp.get('status')) == '200':
            raise AssertionError(resp, content)

        return oauth.Token.from_string(content)


    def authorize(self, request):
        request_token = self.fetch_request_token()

        request.session['request_token'] = {request_token.key: request_token.secret}
        request.session['next_url'] = request.GET.get(REDIRECT_FIELD_NAME, settings.LOGIN_REDIRECT_URL)
        request.session.save()

        return '{0}?oauth_token={1}'.format(
            self.authorization_url, request_token.key
        )


    def get(self, url, **kwargs):
        from warnings import warn;
        warn(
            'Existe um bug no provider do python-oauth2 quando requisições utilizando GET são utilizadas. '
            'Você provavelmente vai precisar usar um POST.'
        )
        resp, content = self.request(url, method='GET', **kwargs)

        return resp, content


    def post(self, url, **kwargs):
        resp, content = self.request(url, method='POST', **kwargs)

        return resp, content



# -*- coding: utf-8 -*-
from django.conf import settings

def hosts(request):
    return {
        'PASSAPORTE_WEB_HOST': settings.PASSAPORTE_WEB['HOST'],
        'APPLICATION_HOST': settings.APPLICATION_HOST
    }




# -*- coding: utf-8 -*-
from django.forms.util import ErrorDict, ErrorList
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from django.core.urlresolvers import reverse

def prepare_form_errors(error_dict):
    if 'field_errors' in error_dict:
        error_dict.update(**error_dict['field_errors'])
    return ErrorDict((k, ErrorList(v)) for (k, v) in error_dict.items())


def get_account_module():
    try:
        module_name =  settings.SERVICE_ACCOUNT_MODULE
        app_name, model_name = module_name.split('.')

        models_name = '%s.models' % app_name
        models = __import__(models_name, fromlist=[models_name])

        return getattr(models, model_name)

    except AttributeError:
        return

    except (ValueError, ImportError):
        raise ImproperlyConfigured(
            'settings.SERVICE_ACCOUNT_MODULE: %s could not be imported' % module_name
        )

def reverse_with_host(namespace, *args, **kwargs):
    path = reverse(namespace, *args, **kwargs)
    if path.startswith(settings.APPLICATION_HOST):
        return path
    else:
        return '{0}{1}'.format(settings.APPLICATION_HOST, path)



# -*- coding: utf-8 -*-
from datetime import datetime

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

try:
    from django.contrib.localflavor.br import forms as br_forms
except ImportError:
    from localflavor.br import forms as br_forms

from timezones import PRETTY_TIMEZONE_CHOICES
from backend import MyfcidAPIBackend

__all__ = [
    'Html5Form',
    'RegistrationForm', 'IdentityAuthenticationForm',
    'IdentityInformationForm', 'IdentityProfileForm',
]

class Html5Form(forms.Form):

    def __init__(self, *args, **kwargs):
        super(Html5Form, self).__init__(*args, **kwargs)

        for item in self.fields.values():
            item.widget.attrs["placeholder"] = item.label


class RegistrationForm(Html5Form):
    """
    Form for registering a new user account.

    Validates that the email is not already registered, and
    requires the password to be entered twice to catch typos.

    Subclasses should feel free to add any additional validation they
    need, but should avoid defining a ``save()`` method -- the actual
    saving of collected user data is delegated to the active
    registration backend.

    """
    first_name = forms.CharField(
        label=_('First Name'), max_length=50, required=False
    )
    last_name = forms.CharField(
        label=_('Last Name'), max_length=100, required=False
    )
    email = forms.EmailField(
        label=_("Email Address"), max_length=75, required=True
    )

    password = forms.CharField(
        label=_("Password"),
        widget=forms.PasswordInput,
        min_length=6, max_length=50
    )
    password2 = forms.CharField(
        label=_("Password (again)"),
        widget=forms.PasswordInput,
        min_length=6, max_length=50
    )

    cpf = br_forms.BRCPFField(
        required=False,
        label=_("CPF")
    )
    tos = forms.BooleanField(
        label=_(u'I have read and agree to the Terms of Service'),
        required=True,
        error_messages={
            'required': _(u"Você precisa ler e concordar com os termos de serviço")
        }
    )


    def clean(self):
        """
        Verifiy that the values entered into the two password fields
        match. Note that an error here will end up in
        ``non_field_errors()`` because it doesn't apply to a single
        field.

        """
        if 'password' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_(u"As senhas informadas são diferentes."))
        return self.cleaned_data


class IdentityAuthenticationForm(Html5Form):
    """
    Base class for authenticating users. Extend this to get a form that accepts
    email/password logins.
    """
    email = forms.EmailField(label=_(u"Email"), max_length=75, required=True)
    password = forms.CharField(label=_(u"Password"), widget=forms.PasswordInput)

    def __init__(self, request=None, *args, **kwargs):
        """
        If request is passed in, the form will validate that cookies are
        enabled. Note that the request (a HttpRequest object) must have set a
        cookie with the key TEST_COOKIE_NAME and value TEST_COOKIE_VALUE before
        running this validation.
        """
        self.request = request
        self.user_cache = None
        super(IdentityAuthenticationForm, self).__init__(*args, **kwargs)

    def clean(self):
        email = self.cleaned_data.get('email')
        password = self.cleaned_data.get('password')

        if email and password:
            self.user_cache = MyfcidAPIBackend().authenticate(
                email=email, password=password
            )
            if self.user_cache is None:
                raise forms.ValidationError(_(u"Por favor digite seu e-mail e senha. O sistema diferencia minúsculas de maiúsculas."))
            elif not self.user_cache.is_active:
                raise forms.ValidationError(_(u"Esta conta está inativa."))

        # TODO: determine whether this should move to its own method.
        if self.request:
            if not self.request.session.test_cookie_worked():
                raise forms.ValidationError(_(u"Parece que seu navegador não aceita cookies. Por favor habilite os cookies."))

        return self.cleaned_data

    def get_user_id(self):
        if self.user_cache:
            return self.user_cache.id
        return None

    def get_user(self):
        return self.user_cache


class IdentityInformationForm(Html5Form):
    """
    Form for updating user's data. All fields compulsory. Email and password
    not changeable here.
    """
    first_name = forms.CharField(
        max_length=50,
        widget=forms.TextInput(),
        required=False,
        label=_("Nome")
    )
    last_name = forms.CharField(
        max_length=100,
        widget=forms.TextInput(),
        required=False,
        label=_("Sobrenome")
    )
    send_myfreecomm_news = forms.BooleanField(
        label=_(u"Quero receber novidades e promoções do Passaporte Web"),
        required=False,
        widget=forms.CheckboxInput(attrs={'class': 'form-checkbox-input'}),
    )
    send_partner_news = forms.BooleanField(
        label=_(u"Quero receber novidades e promoções de parceiros"),
        required=False,
        widget=forms.CheckboxInput(attrs={'class': 'form-checkbox-input'}),
    )
    cpf = br_forms.BRCPFField(
        required=False,
        label=_("CPF")
    )


class IdentityProfileForm(Html5Form):

    nickname = forms.CharField(
        max_length=100,
        widget=forms.TextInput(),
        required=False,
        label=_("Apelido")
    )
    birth_date = forms.DateTimeField(
        widget=forms.DateInput(),
        input_formats = ('%d/%m/%Y', '%d/%m/%y', '%d-%m-%Y', '%d-%m-%y', '%Y-%m-%d'),
        required=False,
        label=_("Data de nascimento"),
        error_messages={
            'invalid': _(u'Enter a valid date.'),
        },
    )
    gender = forms.ChoiceField(
        choices = (
            ('-', _(u"Não informado")),
            ('M', _(u"Masculino")),
            ('F', _(u"Feminino"))
        ),
        required=False,
        label=_(u"Gênero")
    )
    company = forms.CharField(
        max_length=100,
        widget=forms.TextInput(),
        required=False,
        label=_("Empresa")
    )
    position = forms.CharField(
        max_length=100,
        widget=forms.TextInput(),
        required=False,
        label=_("Cargo")
    )
    profession = forms.CharField(
        max_length=100,
        widget=forms.TextInput(),
        required=False,
        label=_(u"Profissão")
    )
    bio = forms.CharField(
        max_length=1024,
        widget=forms.Textarea(),
        required=False,
        label=_(u"Sobre você")
    )
    city = forms.CharField(
        max_length=50,
        widget=forms.TextInput(),
        required=False,
        label=_("Cidade")
    )
    state = br_forms.BRStateChoiceField(
        label=_("Estado"), required=False,
    )
    country = forms.CharField(
        max_length=50,
        widget=forms.TextInput(),
        required=False,
        label=_(u"País")
    )
    language = forms.ChoiceField(
        required=False,
        label=_("Idioma nativo"),
        choices=settings.LANGUAGES,
    )
    timezone = forms.ChoiceField(
        required=False,
        label=_(u"Fuso horário"),
        choices=PRETTY_TIMEZONE_CHOICES,
    )

    def clean_birth_date(self):
        data = self.cleaned_data.get('birth_date', None)
        if data and (not datetime(1900, 1, 1) <= data <= datetime(9999, 12, 31)):
            raise forms.ValidationError(
                self.fields['birth_date'].error_messages['invalid']
            )

        return data



# -*- coding: utf-8 -*-
from identity_client.signals import pre_identity_authentication
from identity_client.utils import get_account_module

def update_identity_accounts(sender, identity, user_data, **kwargs):

    serviceAccount = get_account_module()
    accounts = user_data.get('accounts', None)

    if (serviceAccount is not None) and (accounts is not None):
        serviceAccount.update_user_accounts(identity, accounts)

    return


def dissociate_old_identity_accounts(sender, identity, user_data, **kwargs):

    serviceAccount = get_account_module()
    accounts = user_data.get('accounts', None)

    if (serviceAccount is not None) and (accounts is not None):
        serviceAccount.remove_stale_accounts(identity, accounts)

    return


pre_identity_authentication.connect(
    update_identity_accounts, dispatch_uid="update_identity_accounts"
)
pre_identity_authentication.connect(
    dissociate_old_identity_accounts, dispatch_uid="dissociate_old_identity_accounts"
)



# -*- coding: utf-8 -*-
from identity_client import PERSISTENCE_MODULE

Identity = PERSISTENCE_MODULE.models.Identity
ServiceAccount = PERSISTENCE_MODULE.models.ServiceAccount
AccountMember = PERSISTENCE_MODULE.models.AccountMember



# -*- coding: utf-8 -*-
import logging

from datetime import datetime
from passaporte_web.main import ServiceAccount as RemoteServiceAccount

from mongoengine import *
from mongoengine.queryset import Q

from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth.models import SiteProfileNotAvailable

from identity_client.client_api_methods import APIClient
from identity_client.decorators import asyncmethod

class Identity(Document):
    """
    Myfc ID Users within the Django authentication system are represented by
    this model.

    email is required. Other fields are optional.
    """
    first_name = StringField(max_length=50)
    last_name = StringField(max_length=100)
    email = StringField()
    uuid = StringField(max_length=36, unique=True)
    id_token = StringField(max_length=48, default=None)
    is_active = BooleanField(default=False)

    meta = {'allow_inheritance': True}

    def __unicode__(self):
        return self.email

    def is_anonymous(self):
        """
        Always returns False. This is a way of comparing User objects to
        anonymous users.
        """
        return False

    def is_authenticated(self):
        """
        Always return True. This is a way to tell if the user has been
        authenticated in templates.
        """
        return True

    def is_staff(self):
        """
        Always returns False.
        """
        return False

    def get_full_name(self):
        "Returns the first_name plus the last_name, with a space in between."
        full_name = u'%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def set_password(self, raw_password):
        raise NotImplementedError

    def check_password(self, raw_password):
        raise NotImplementedError

    def set_unusable_password(self):
        raise NotImplementedError

    def has_usable_password(self):
        return False

    def get_and_delete_messages(self):
        messages = []
        for m in self.message_set.all():
            messages.append(m.message)
            m.delete()
        return messages

    def email_user(self, subject, message, from_email=None):
        "Sends an e-mail to this User."
        from django.core.mail import send_mail
        send_mail(subject, message, from_email, [self.email])

    def get_profile(self):
        """
        Returns site-specific profile for this user. Raises
        SiteProfileNotAvailable if this site does not allow profiles.
        """
        if not hasattr(self, '_profile_cache'):
            from django.conf import settings
            if not getattr(settings, 'AUTH_PROFILE_MODULE', False):
                raise SiteProfileNotAvailable
            try:
                app_label, model_name = settings.AUTH_PROFILE_MODULE.split('.')
                model = models.get_model(app_label, model_name)
                self._profile_cache = model._default_manager.using(self._state.db).get(user__id__exact=self.id)
                self._profile_cache.user = self
            except (ImportError, ImproperlyConfigured):
                raise SiteProfileNotAvailable
        return self._profile_cache

    def _get_message_set(self):
        import warnings
        warnings.warn('The user messaging API is deprecated. Please update'
                      ' your code to use the new messages framework.',
                      category=PendingDeprecationWarning)
        return self._message_set
    message_set = property(_get_message_set)


class AccountMember(EmbeddedDocument):
    identity = ReferenceField(Identity, required=True, dbref=True)
    roles = ListField(StringField(max_length=36))

    def set_roles(self, roles):
        roles = set(roles)
        self.roles = list(roles)

    def __unicode__(self):
        return u'%s - [%s]' % (self.identity.email, ','.join(self.roles))


class ServiceAccount(Document):
    name = StringField(max_length=256, required=True)
    uuid = StringField(max_length=36, required=True)
    plan_slug = StringField(max_length=50, required=True)
    members = ListField(EmbeddedDocumentField(AccountMember))
    expiration = DateTimeField(required=False)
    url = StringField(max_length=1024, required=False)

    meta = {'allow_inheritance': True}

    def __unicode__(self):
        return self.name

    @queryset_manager
    def active(cls, qset):
        return qset.filter(Q(expiration=None)|Q(expiration__gte=datetime.now()))

    @classmethod
    def for_identity(cls, identity, include_expired=False):
        if include_expired:
            qset = cls.objects
        else:
            qset = cls.active

        return qset.filter(members__identity=identity)

    @classmethod
    def refresh_accounts(cls, identity, **kwargs):

        accounts = cls.pull_remote_accounts(identity, **kwargs)
        cls.update_user_accounts(identity, accounts)
        cls.remove_stale_accounts(identity, accounts)

    @classmethod
    def pull_remote_accounts(cls, identity, **kwargs):
        status_code, accounts, error = APIClient.fetch_user_accounts(identity.uuid, **kwargs)

        default_roles = []
        if 'role' in kwargs:
            default_roles.append(kwargs['role'])

        if error:
            return []
        else:
            return [dict(
                uuid = item.get('account_data', item)['uuid'],
                name = item.get('account_data', item)['name'],
                expiration = item.get('expiration'),
                plan_slug = item.get('plan_slug', 'UNKNOWN'),
                url = item.get('url'),
                roles = item.get('roles', default_roles),
            ) for item in accounts]

    @classmethod
    def update_user_accounts(cls, identity, accounts):
        for item in accounts:
            uuid = item['uuid']
            name = item['name']
            expiration = item.get('expiration')
            plan_slug = item['plan_slug']
            url = item['url']
            roles = item['roles']

            try:
                account = cls.objects.get(uuid=uuid)
            except cls.DoesNotExist:
                account = cls(uuid=uuid)

            account.name = name
            account.plan_slug = plan_slug
            account.url = url

            if expiration:
                new_expiration = datetime.strptime(expiration, '%Y-%m-%d')
                account.update_expiration(new_expiration)
            else:
                account.update_expiration(None)

            try:
                account.add_member(identity, roles)
                account.save()
            except Exception, e:
                message = 'Error updating accounts for identity %s (%s): %s <%s>'
                logging.error(message, identity.email, identity.uuid, e, type(e))

            message = 'Identity %s (%s) at account %s (%s) members list'
            logging.info(message, identity.email, identity.uuid, account.name, account.uuid)

    @classmethod
    def remove_stale_accounts(cls, identity, accounts):
        current_uuids = [item['uuid'] for item in accounts]

        cached_associations = cls.for_identity(identity, include_expired=True)
        stale_accounts = [account for account in cached_associations if account.uuid not in current_uuids]

        for account in stale_accounts:
            account.remove_member(identity)
            account.save()
            message = 'Identity %s (%s) was removed from account %s (%s) members list.'
            logging.info(message, identity.email, identity.uuid, account.name, account.uuid)

    @property
    def is_active(self):
        return (self.expiration is None) or (self.expiration >= datetime.now())

    @property
    def members_count(self):
        return len(self.members)

    def get_member(self, identity):
        for item in self.members:
            if item.identity == identity:
                return item

        return None

    def add_member(self, identity, roles):
        new_member = self.get_member(identity)
        if new_member is None:
            new_member = AccountMember(identity=identity)
            self.members.append(new_member)

        new_member.set_roles(roles)
        self.save()

        return new_member

    def remove_member(self, identity):
        member = self.get_member(identity)
        if member is None:
            return self

        self.members.remove(member)
        self.save()

        return self

    def clear_members(self):
        self.members = []
        self.save()

    def update_expiration(self, new_expiration):
        self.expiration = new_expiration

    @asyncmethod
    def send_notification(self, body, **kwargs):
        return RemoteServiceAccount.load(
            self.url, token=APIClient.api_user, secret=APIClient.api_password
        ).send_notification(body, **kwargs)



# -*- coding: utf-8 -*-
from mock import Mock, patch

from django.test import TestCase
from django.conf import settings
from django.core.urlresolvers import reverse

from identity_client import middleware

enabled_mock = Mock()
enabled_mock.P3P_COMPACT = 'CP=ANY P3P POL ICY'

disabled_mock = Mock()
disabled_mock.P3P_COMPACT = None

class TestP3PMiddleware(TestCase):

    @patch.object(middleware, 'settings', enabled_mock)
    def test_p3p_header_is_added(self):
        response = self.client.get(reverse('auth_login'))
        self.assertTrue('P3P' in response)
        self.assertEquals(response['P3P'], 'CP=ANY P3P POL ICY')


    @patch.object(middleware, 'settings', disabled_mock)
    def test_p3p_header_is_not_added_is_P3P_COMPACT_not_in_settings(self):
        response = self.client.get(reverse('auth_login'))
        self.assertFalse('P3P' in response)



# -*- coding: utf-8 -*-
import logging
import oauth2 as oauth
from httplib2 import HttpLib2Error

from django.http import HttpResponseServerError, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.conf import settings

from identity_client.sso.client import SSOClient


__all__ = ['oauth_callback', ]


def oauth_callback(view):

    def decorated(request, *args, **kwargs):

        if 'request_token' not in request.session:
            # O fluxo de autenticação ainda não foi iniciado

            # Salvar a url do callback na sessão, ignorando sua querystring
            request.session['callback_url'] = u'{0}{1}'.format(
                settings.APPLICATION_HOST, request.path
            )
            request.session.save()

            return HttpResponseRedirect(reverse('sso_consumer:request_token'))

        else:
            # O fluxo de autenticação já foi iniciado

            if  getattr(request, 'access_token', None) is None:
                # Obter access token
                oauth_token = request.GET.get('oauth_token')
                oauth_verifier = request.GET.get('oauth_verifier')

                request_token = request.session['request_token']
                secret = request_token[oauth_token]

                request_token = oauth.Token(key=oauth_token, secret=secret)
                request_token.set_verifier(oauth_verifier)

                try:
                    # Adicionar access token à sessão
                    access_token = SSOClient(request_token).fetch_access_token()
                    request.session['access_token'] = {
                        access_token.key: access_token.secret
                    }
                    request.session.save()

                    # E à request
                    request.access_token = access_token

                except AssertionError, e:
                    resp, content = e.args

                    message = "Could not fetch access token. Response was {0} - {1}".format(
                        resp.get('status'), content
                    )
                    logging.error(message)
                    return HttpResponseServerError(content=message)

                except ValueError, e:
                    message = "Invalid access token"
                    logging.error(message)
                    return HttpResponseServerError(content=message)

                except HttpLib2Error, e:
                    message = "Could not fetch access token, communication failure: {0} ({1})".format(
                        e, type(e)
                    )
                    logging.error(message)
                    return HttpResponseServerError(status=502, content=message)

                except Exception, e:
                    message = "Could not fetch access token, unknown error: {0} ({1})".format(
                        e, type(e)
                    )
                    logging.error(message)
                    return HttpResponseServerError(content=message)

            return view(request, *args, **kwargs)

    return decorated



# -*- coding: utf-8 -*-
import json

from mock import patch

from django.conf import settings
from django.contrib.auth.models import AnonymousUser

import identity_client
from identity_client.models import Identity
from identity_client.backend import MyfcidAPIBackend, get_user
from identity_client.utils import get_account_module
from identity_client.tests.helpers import MyfcIDTestCase as TestCase
from identity_client.client_api_methods import APIClient

__all__ = ['TestMyfcidApiBackend', 'TestGetUser', 'TestFetchUserData']

test_user_email = 'identity_client@disposableinbox.com'
test_user_password = '*SudN7%r$MiYRa!E'
test_user_uuid = 'c3769912-baa9-4a0c-9856-395a706c7d57'


class TestMyfcidApiBackend(TestCase):

    def test_successful_auth(self):
        # Autenticar um usuário
        with identity_client.tests.use_cassette('myfcid_api_backend/success'):
            identity = MyfcidAPIBackend().authenticate(test_user_email, test_user_password)

        # Checar se o usuário foi autenticado corretamente
        self.assertNotEqual(identity, None)
        self.assertEquals(identity.first_name, 'Identity')
        self.assertEquals(identity.last_name, 'Client')
        self.assertEquals(identity.email, test_user_email)
        self.assertEquals(identity.user_data, {
            u'authentication_key': u'$2a$12$nA3ad2y5aSBlg80K9ekbNuvnRO1OI1WUKZyoJqWEhk.PQpD8.6jkS',
            u'email': u'identity_client@disposableinbox.com',
            u'first_name': u'Identity',
            u'id_token': u'729dd3a15cf03a80024d0986deee9ae91fdd5d834fabf6f9',
            u'is_active': True,
            u'last_name': u'Client',
            u'notifications': {u'count': 0, u'list': u'/notifications/api/'},
            u'profile_url': u'/accounts/api/identities/c3769912-baa9-4a0c-9856-395a706c7d57/profile/',
            u'send_myfreecomm_news': True,
            u'send_partner_news': True,
            u'update_info_url': u'/accounts/api/identities/c3769912-baa9-4a0c-9856-395a706c7d57/',
            u'uuid': u'c3769912-baa9-4a0c-9856-395a706c7d57'
        })

        # Checar se o backend foi setado corretamente
        self.assertEquals(
            identity.backend,
            '%s.%s' % (MyfcidAPIBackend.__module__, 'MyfcidAPIBackend')
        )


    def test_failed_auth(self):
        # Autenticar um usuário
        with identity_client.tests.use_cassette('myfcid_api_backend/wrong_password'):
            identity = MyfcidAPIBackend().authenticate(test_user_email, 'senha errada')

        # Garantir que o usuario não foi autenticado
        self.assertEquals(identity, None)


    def test_auth_updates_user(self):
        # Create a user
        user = Identity.objects.create(uuid=test_user_uuid, email='vai_mudar@email.com')

        # Autenticar um usuário
        with identity_client.tests.use_cassette('myfcid_api_backend/success'):
            identity = MyfcidAPIBackend().authenticate(test_user_email, test_user_password)

        # Checar se os dados do usuário foram atualizados
        self.assertEquals(identity.first_name, 'Identity')
        self.assertEquals(identity.last_name, 'Client') 
        self.assertEquals(identity.email, test_user_email)
        self.assertEquals(identity.uuid, test_user_uuid)


    @patch.object(settings, 'SERVICE_ACCOUNT_MODULE', 'identity_client.ServiceAccount')
    def test_create_local_identity_creates_user_accounts(self):
        # Obter dados do usuário
        with identity_client.tests.use_cassette('fetch_identity_data/success_with_accounts'):
            response = APIClient.fetch_identity_data(uuid=test_user_uuid)
            status_code, content, error = response

        # Criar usuário local
        identity = MyfcidAPIBackend().create_local_identity(content)

        # 1 conta deve ter sido criada
        serviceAccountModel = get_account_module()
        accounts = serviceAccountModel.for_identity(identity)
        self.assertEquals(accounts.count(), 3)


    @patch.object(settings, 'SERVICE_ACCOUNT_MODULE', 'identity_client.ServiceAccount')
    def test_create_local_identity_removes_user_from_old_accounts(self):
        # Obter dados do usuário
        with identity_client.tests.use_cassette('fetch_identity_data/success'):
            response = APIClient.fetch_identity_data(uuid=test_user_uuid)
            status_code, content, error = response

        # Criar usuário local
        identity = MyfcidAPIBackend().create_local_identity(content)

        # 5 contas devem ter sido criadas
        serviceAccountModel = get_account_module()
        accounts = serviceAccountModel.for_identity(identity)
        self.assertEquals(accounts.count(), 5)

        # Obter os dados do usuário, desta vez sem accounts
        with identity_client.tests.use_cassette('fetch_identity_data/success_with_accounts'):
            response = APIClient.fetch_identity_data(uuid=test_user_uuid)
            status_code, content, error = response

        # Criar usuário local
        identity = MyfcidAPIBackend().create_local_identity(content)

        # O usuário deve ter sido dissociado da account
        serviceAccountModel = get_account_module()
        accounts = serviceAccountModel.for_identity(identity)
        self.assertEquals(accounts.count(), 3)


    def test_accounts_creation_fails_if_settings_are_wrong(self):
        # Obter dados do usuário
        with identity_client.tests.use_cassette('fetch_identity_data/success_with_accounts'):
            response = APIClient.fetch_identity_data(uuid=test_user_uuid)
            status_code, content, error = response

        # Criar usuário local
        with patch.object(settings, 'SERVICE_ACCOUNT_MODULE', 'unknown_app.UnknownModel'):
            identity = MyfcidAPIBackend().create_local_identity(content)

        self.assertTrue(identity is not None)

        # Nenhuma conta foi criada
        serviceAccountModel = get_account_module()
        accounts = serviceAccountModel.for_identity(identity)
        self.assertEquals(accounts.count(), 0)


    def test_auth_user_accounts_creation_fails_if_settings_are_missing(self):
        # Obter dados do usuário
        with identity_client.tests.use_cassette('fetch_identity_data/success_with_accounts'):
            response = APIClient.fetch_identity_data(uuid=test_user_uuid)
            status_code, content, error = response

        # Criar usuário local
        with patch.object(settings, 'SERVICE_ACCOUNT_MODULE', None):
            identity = MyfcidAPIBackend().create_local_identity(content)

        self.assertTrue(identity is not None)

        # Nenhuma conta foi criada
        serviceAccountModel = get_account_module()
        accounts = serviceAccountModel.for_identity(identity)
        self.assertEquals(accounts.count(), 0)


    @patch.object(settings, 'SERVICE_ACCOUNT_MODULE', 'identity_client.ServiceAccount')
    def test_authentication_should_not_remove_user_accounts(self):
        # Obter dados do usuário
        with identity_client.tests.use_cassette('fetch_identity_data/success_with_accounts'):
            response = APIClient.fetch_identity_data(uuid=test_user_uuid)
            status_code, content, error = response

        # Criar usuário local
        identity = MyfcidAPIBackend().create_local_identity(content)

        # 1 conta deve ter sido criada
        serviceAccountModel = get_account_module()
        accounts = serviceAccountModel.for_identity(identity)
        self.assertEquals(accounts.count(), 3)

        # Autenticar o usuário
        with identity_client.tests.use_cassette('myfcid_api_backend/success'):
            identity = MyfcidAPIBackend().authenticate(test_user_email, test_user_password)

        # A conta deve continuar existindo
        serviceAccountModel = get_account_module()
        accounts = serviceAccountModel.for_identity(identity)
        self.assertEquals(accounts.count(), 3)


class TestGetUser(TestCase):

    def _create_user(self):
        # Autenticar um usuário
        with identity_client.tests.use_cassette('myfcid_api_backend/success'):
            return MyfcidAPIBackend().authenticate(test_user_email, test_user_password)


    def test_valid_user(self):
        identity = self._create_user()
        user = get_user(userid=identity.id)
        self.assertEquals(user, identity)


    def test_user_not_sent(self):
        user = get_user(userid=None)
        self.assertTrue(isinstance(user, AnonymousUser))


    def test_invalid_user(self):
        user = get_user(userid=42)
        self.assertTrue(isinstance(user, AnonymousUser))


class TestFetchUserData(TestCase):

    def test_fetch_user_data_with_success(self):
        with identity_client.tests.use_cassette('myfcid_api_backend/success'):
            _, user_data, _ = MyfcidAPIBackend().fetch_user_data(test_user_email, test_user_password)

        self.assertEquals(user_data, {
            u'authentication_key': u'$2a$12$nA3ad2y5aSBlg80K9ekbNuvnRO1OI1WUKZyoJqWEhk.PQpD8.6jkS',
            u'email': u'identity_client@disposableinbox.com',
            u'first_name': u'Identity',
            u'id_token': u'729dd3a15cf03a80024d0986deee9ae91fdd5d834fabf6f9',
            u'is_active': True,
            u'last_name': u'Client',
            u'notifications': {u'count': 0, u'list': u'/notifications/api/'},
            u'profile_url': u'/accounts/api/identities/c3769912-baa9-4a0c-9856-395a706c7d57/profile/',
            u'send_myfreecomm_news': True,
            u'send_partner_news': True,
            u'update_info_url': u'/accounts/api/identities/c3769912-baa9-4a0c-9856-395a706c7d57/',
            u'uuid': u'c3769912-baa9-4a0c-9856-395a706c7d57'
        })

    def test_fetch_user_data_failure(self):
        with identity_client.tests.use_cassette('myfcid_api_backend/wrong_password'):
            _, user_data, error = MyfcidAPIBackend().fetch_user_data(test_user_email, 'senha errada')

        self.assertEquals(user_data, None)
        self.assertEquals(error, {'status': 401, 'message': '401 Client Error: UNAUTHORIZED'})



# -*- coding: utf-8 -*-
import setuptools
from os.path import join, dirname

from setuptools.command.test import test as TestCommand
import sys

class Tox(TestCommand):

    user_options = TestCommand.user_options + [
        ('environment=', 'e', "Run 'test_suite' in specified environment")
    ]
    environment = None

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import tox
        if self.environment:
            self.test_args.append('-e{0}'.format(self.environment))
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)

setuptools.setup(
    name="django-passaporte-web",
    version="1.1.6",
    packages=["identity_client"],
    include_package_data=True,  # declarations in MANIFEST.in
    install_requires=open(join(dirname(__file__), 'requirements.txt')).readlines(),
    tests_require=['tox', 'virtualenv>=1.11.2'],
    cmdclass = {'test': Tox},
    test_suite='runtests.runtests',
    author="vitormazzi",
    author_email="vitormazzi@gmail.com",
    url="http://github.com/myfreecomm/django-passaporte-web",
    license="Apache 2.0",
    description="Django client app for Passaporte Web.",
    long_description=open(join(dirname(__file__), "README.rst")).read(),
    keywords="django python passaporteweb",
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
    ],
)
